unit TransBmp3;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus;

type
  TTransBmp3 = class(TGraphicControl)
  private
    FBitmap, FDirtyBitmap: TBitmap;
    FTransColor: TColor;
    FIndex: Integer;
    FList: TImageList;
  protected
    procedure SetBitmap(Bmp: TBitmap);
    procedure SetTransColor(Val: TColor);
    procedure UpdateBmp;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure Paint; override;
  published
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property TransColor: TColor read FTransColor write SetTransColor default clBlack;
    property DragCursor;
    property DragMode;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
  end;

procedure Register;

implementation

constructor TTransBmp3.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBitmap := TBitmap.Create;
  FDirtyBitmap := TBitmap.Create;
  FList := TImageList.Create(Self);
  { Set arbitrary default, otherwise it will be 0x0 }
  Height := 40;
  Width := 40;
end;

destructor TTransBmp3.Destroy;
begin
  FList.Free;
  FBitmap.Free;
  FDirtyBitmap.Free;
  inherited Destroy;
end;

procedure TTransBmp3.SetTransColor(Val: TColor);
begin
  if Val <> FTransColor then
  begin
    FTransColor := Val;
    { Ensure mask is regenerated with new idea of transparent colour }
    UpdateBmp;
    Invalidate;
  end;
end;

procedure TTransBmp3.SetBitmap(Bmp: TBitmap);
begin
  FBitmap.Assign(Bmp);
  if Assigned(Bmp) then
  begin
    Height := Bmp.Height;
    FList.Height := Height;
    Width := Bmp.Width;
    FList.Width := Width;
    { Get default transparent colour (bottom left pixel). The And op }
    { gets rid of leading non-RGB byte which indicates palette index }
    FTransColor := FBitmap.TransparentColor and clWhite;
    UpdateBmp;
  end;
  Invalidate;
end;

{ The original bitmap is altered (transparent }
{ area becomes black) so we use a copy to add }
{ into the list leaving the original intact }
procedure TTransBmp3.UpdateBmp;
begin
    FDirtyBitmap.Assign(FBitmap);
    if FList.Count > 0 then
      FList.Delete(FIndex);
    FIndex := FList.AddMasked(FDirtyBitmap, FTransColor);
end;

procedure TTransBmp3.Loaded;
begin
  inherited Loaded;
  SetBitmap(FBitmap);
end;

procedure TTransBmp3.Paint;
begin
  FList.Draw(Canvas, 0, 0, FIndex);
end;

procedure Register;
begin
  RegisterComponents('Samples', [TTransBmp3]);
end;

end.
